/**********************************************************************
Eternal Story on Temporary Objects

Dmitri Korotkevitch
http://aboutsqlserver.com
dmitri@aboutsqlserver.com

01. Temp Tables Usage
**********************************************************************/
set nocount on
go

use [24HOP_2014]
go

if object_id(N'dbo.ParseIDList') is not null
	drop function dbo.ParseIDList
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where t.name = 'Orders' and s.name = 'dbo'
)
	drop table dbo.Orders
go

create function dbo.ParseIDList(@List varchar(8000))
returns @IDList table
(
	ID int
)
as
begin
	if (IsNull(@List,'') = '')
		return 

	if (right(@List,1) <> ',')
		select @List += ','

	;with CTE(F, L)
	as
	(
		select 1, charindex(',',@List)
		union all
		select L + 1, charindex(',',@List,L + 1)
		from CTE
		where charindex(',',@List,L + 1) <> 0
	)
	insert into @IDList(ID)
		select distinct convert(int,substring(@List,F,L-F))
		from CTE
	option (maxrecursion 0);
	
	return
end
go

create table dbo.Orders
(
	OrderId int not null,
	CustomerId int not null,
	OrderDate date not null,
	Amount money not null,
	Cancelled bit not null
		constraint DEF_Orders_Cancelled
		default 0,
	PendingCancellation bit not null
		constraint DEF_Orders_PendingCancellation
		default 0,
	Paid bit not null
		constraint DEF_Orders_Paid
		default 0,
	Backordered bit not null
		constraint DEF_Orders_Backordered
		default 0,
	EstimatedFulfillDate date null,
	Placeholder char(100),
	Comments varchar(max),

	constraint PK_Orders
	primary key clustered(OrderId)
);

with N1(C) as (select 0 union all select 0) -- 2 rows    
,N2(C) as (select 0 from N1 as T1 CROSS JOIN N1 as T2) -- 4 rows    
,N3(C) as (select 0 from N2 as T1 CROSS JOIN N2 as T2) -- 16 rows    
,N4(C) as (select 0 from N3 as T1 CROSS JOIN N3 as T2) -- 256 rows    
,N5(C) as (select 0 from N4 as T1 CROSS JOIN N4 as T2) -- 65,536 rows    
,IDs(ID) as (select row_number() over (order by (select NULL)) from N5)    
	insert into dbo.Orders(OrderId, CustomerId, Amount, OrderDate, Paid)            
		select ID, ID % 250 + 1, Id % 50, dateadd(day,-ID % 100, getDate()), 1
		from IDs;

create index IDX_Orders_CustomerId on dbo.Orders(CustomerId)
go



/**********************************************************
Improving Cardinality Estimates

You can use the following code to generate CSV list

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 CROSS JOIN N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 CROSS JOIN N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 CROSS JOIN N3 as T2) -- 256 rows
,IDs(ID) as (select row_number() over (order by (select NULL)) from N4)
select @List = convert(varchar(8000),
		(
			select ID as [text()], ',' as [text()]
			from IDs
			where ID <= 250
			for xml path('')
		)
	) 
**********************************************************/
declare
	@List varchar(8000) = '1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,249,250'

set statistics io, time on

select sum(o.Amount)
from dbo.Orders o join dbo.ParseIDList(@List) l on
	o.CustomerID = l.ID
     
set statistics io, time off
go

declare
	@List varchar(8000) = '1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,249,250'

set statistics io, time on

create table #Customers(ID int not null primary key)
insert into #Customers(ID)
	select ID from dbo.ParseIDList(@List)
	
select sum(o.Amount)
from dbo.Orders o join #Customers c on
	o.CustomerID = c.ID

set statistics io, time off
drop table #Customers
go



/**********************************************************
Reducing locking in the system
**********************************************************/
update dbo.Orders set PendingCancellation = 1 where OrderId in (10000,20000,30000,40000,50000,60000);
go

update dbo.Orders
set
	Cancelled = 1
where
	(PendingCancellation = 1) or 
	(Paid = 0 and OrderDate < dateadd(day,-3,getDate())) or
	(BackOrdered = 1 and EstimatedFulfillDate > dateadd(day,-2,getDate()))
go





create table #OrderIds(OrderId int not null primary key(OrderId))
insert into #OrderIds(OrderId)
	select OrderId
	from dbo.Orders
	where
		(PendingCancellation = 1) or 
		(Paid = 0 and OrderDate < dateadd(day,-3,getDate())) or
		(BackOrdered = 1 and EstimatedFulfillDate > dateadd(day,-2,getDate()))

update dbo.Orders
set
	Cancelled = 1
where
	OrderId in (select OrderId from #OrderIds);

drop table #OrderIds
go


