/**********************************************************************
Eternal Story on Temporary Objects

Dmitri Korotkevitch
http://aboutsqlserver.com
dmitri@aboutsqlserver.com

02. Table variables: Cardinality and Rollback support
**********************************************************************/

set nocount on
go


/* Statement-level rollback */
declare
	@T table(ID int not null primary key)

-- Success
insert into @T(ID) values(1)

-- Error: primary key violation
insert into @T(ID) values(2),(3),(3)

-- 1 row
select * from @T




/* Transaction rollback */
create table #Errors
(
	RecId int not null,
	[Error] nvarchar(512) not null,

	primary key(RecId)
)

declare
	@Errors table
	(
		RecId int not null,
		[Error] nvarchar(512) not null,

		primary key(RecId)
	)

begin tran
	insert into #Errors(RecId, [Error])
	values
		(11,'Price mistake'),  
		(42,'Insufficient stock')

	insert into @Errors(RecId, [Error])
	values
		(11,'Price mistake'),  
		(42,'Insufficient stock')
rollback

select RecId, [Error] from #Errors
select RecId, [Error] from @Errors
go



-- Cardinality
create table #Orders(OrderId int not null primary key);
insert into #Orders(OrderId)
	select OrderId from [24HOP_2014].dbo.Orders
go

/* Check estimated number of rows for CI scan */
declare 
	@Orders table(OrderId int not null primary key)
insert into @Orders(OrderId)
	select OrderId from #Orders;

select count(*) from #Orders
select count(*) from @Orders
select count(*) from @Orders option (recompile)
go



declare 
	@Orders table(OrderId int not null primary key)
insert into @Orders(OrderId)
	select OrderId from #Orders;

select count(*) from #Orders where OrderID > 0
select count(*) from @Orders where OrderID > 0
select count(*) from @Orders where OrderID > 0 option (recompile)
go



