/**********************************************************************
Eternal Story on Temporary Objects

Dmitri Korotkevitch
http://aboutsqlserver.com
dmitri@aboutsqlserver.com

06. Memory Grants: Inaccurate statistics and row size estimations
**********************************************************************/
set nocount on
use [24HOP_2014]
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where t.name = 'MemoryGrantDemo' and s.name = 'dbo'
)
	drop table dbo.MemoryGrantDemo
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where t.name = 'Data1' and s.name = 'dbo'
)
	drop table dbo.Data1
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where t.name = 'Data2' and s.name = 'dbo'
)
	drop table dbo.Data2
go


/* Inaccurate statistics and Memory Grants */
create table dbo.MemoryGrantDemo
(
	ID int not null,
	Col int not null,
	Placeholder char(8000)
);

create unique clustered index IDX_MemoryGrantDemo_ID
on dbo.MemoryGrantDemo(ID);

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 CROSS JOIN N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 CROSS JOIN N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 CROSS JOIN N3 as T2) -- 256 rows
,N5(C) as (select 0 from N4 as T1 CROSS JOIN N4 as T2) -- 65,536 rows
,IDs(ID) as (select row_number() over (order by (select NULL)) from N5)
insert into dbo.MemoryGrantDemo(ID,Col,Placeholder)
	select ID, ID % 100, convert(char(100),ID)
	from IDs;

create nonclustered index IDX_MemoryGrantDemo_Col 
on dbo.MemoryGrantDemo(Col);



-- Adding more rows for Col = 1000
;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 CROSS JOIN N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 CROSS JOIN N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 CROSS JOIN N3 as T2) -- 256 rows
,N5(C) as (select 0 from N4 as T1 CROSS JOIN N2 as T2) -- 1,024 rows
,IDs(ID) as (select row_number() over (order by (select NULL)) from N5)
insert into dbo.MemoryGrantDemo(ID,Col,Placeholder)
	select 100000 + ID, 1000, convert(char(100),ID)
	from IDs
	where ID <= 656;

dbcc show_statistics('dbo.MemoryGrantDemo','IDX_MemoryGrantDemo_Col')
go


declare
	@X int
    
set statistics time on
select @X = ID from dbo.MemoryGrantDemo where Col = 1 order by Placeholder
select @X = ID from dbo.MemoryGrantDemo where Col = 1000 order by Placeholder
set statistics time off
go




/* Row Size Estimations and Memory Grants */
create table dbo.Data1 
( 
	ID int not null, 
	Value varchar(100) not null, 
	constraint PK_Data1 
	primary key clustered(ID) 
); 

create table dbo.Data2 
( 
	ID int not null, 
	Value varchar(200) not null, 
	constraint PK_Data2 
	primary key clustered(ID) 
); 

;with N1(C) as (select 0 union all select 0) -- 2 rows 
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows 
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows 
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) -- 256 rows 
,N5(C) as (select 0 from N4 as T1 cross join N4 as T2 ) -- 65,536 rows 
,Nums(Num) as (select row_number() over (order by (select null)) from N5) 
insert into dbo.Data1(ID, Value) 
	select Num , replicate('0',100) 
	from Nums; 

insert into dbo.Data2(ID, Value) 
	select ID, Value from dbo.Data1; 

declare 
	@V varchar(200) 

select @V = Value 
from dbo.Data1 
where ID < 40425
order by Value, ID desc 

select @V = Value 
from dbo.Data2 
where ID < 40425 
order by Value, ID desc 

